# frozen_string_literal: true

# Legende
# Les script a plusieurs parties indique que les bash (Batch dans l'app),
# peuvent s'initier avec differents types de source
# Le besoin est donc d'isoler la logique associee au modele actuel
# ne pas oublier d'indexer les champs simples

module Hermes
  # Used for enterprise and enterprises sub-entities
  class Entity
    # tous les champs qui sont non décrits ou avec absence de connaissance fonctionnel ont été
    # tirés des XSD et / ou XML des données d'origines
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::AlsIdConcern
    include Hermes::Concerns::Status
    include ::Concerns::Geolocalizable
    
    # champs de dépense de points à prendre en compte au before_update
    SPENT_ON_FIELDS = %w[
      spent_on_completed_requests
      spent_on_ongoing_requests
      spent_on_other
    ].freeze

    field :test_created_at, type: DateTime

    field :title, type: String
    field :siret, type: String
    field :siren, type: String

    field :is_root, type: Boolean, default: true                        # censé être utilisé pour identifier l'entity comme racine de parenté
    field :root_id, type: BSON::ObjectId                                # censé être utilisé pour identifier le parent direct d'une entity

    field :automated_request_validations, type: Boolean, default: false # non décrit, anticipé pour la gestion automatique de la validation de demandes salarié
    field :delegated_validations, type: Boolean, default: false         # pareil que pour le précédent
    field :priorities_enabled, type: Boolean, default: false            # non décrit, en rapport avec la gestion des points de priorité je supposes

    field :in_peec, type: Boolean, default: false                       # non décrit, censé être utilisé pour identifier les entreprises associé au programme peec
    field :achieved_goal, type: Boolean                                 # non décrit, censé gérer l'atteinte ou non des objectifs de l'entreprise

    field :spent_on_completed_requests, type: Integer, default: 0       # permet de stocker et suivre les points de priorité dépensés sur les demandes finalisées
    field :spent_on_ongoing_requests, type: Integer, default: 0         # permet de stocker et suivre les points de priorité dépensés sur les demandes en cours de réalisation
    field :spent_on_other, type: Integer, default: 0                    # ajouté pour identifier un mai-fonctionnement, tout point dépensé qui est assigné à ce champ
                                                                        # indique un problème de gestion des status sur la demande concernée

    field :annual_stock, type: Integer, default: 8500                   # stock de points de priorité annuels de l'entreprise (aucune gestion de la réinitialisation car non décrite)
    field :total_spent, type: Integer, default: 0                       # total dépensé (somme des trois champs spent_on)
    field :points_balance, type: Integer, default: 0                    # balance des points de priorités restant à l'entreprise

    field :supervisor_group_id, type: BSON::ObjectId                    # group superviseur (lecture des objets associés - demandes / offres) de l'entreprise (auto-généré)
    field :manager_group_id, type: BSON::ObjectId                       # group manageur (écriture des objets associés - demandes / offres) de l'entreprise (auto-généré)

    alias_attribute :district, :city
    #has_one :image, class_name: '::Doc'
    #has_one :contact_group, class_name: 'Hermes::ContactGroup'
    has_many :applicants, class_name: 'Hermes::Applicant'
    #has_and_belongs_to_many :housing_offers, class_name: 'Hermes::HousingOffer'
    # TODO: handle also with housing_offer_ids=
    has_many :housing_entity_offers, class_name: 'Hermes::HousingEntityOffer'
    belongs_to(:lessor, class_name: 'Hermes::Lessor', index: true)
    belongs_to(:group, class_name: 'Group', index: true)

    index(applicant_ids: 1)
    index(housing_entity_offer_ids: 1)

    index(status: 1)
    index(status_id: 1)
    index(status_updated_at: 1)
    index(old_status: 1)

    before_create :before_create_actions
    before_update :before_actions # https://dev.hermes.openstack.local/admin/families/5d3ea80c4ac06e63808d0fc5/mapping
    after_save :after_actions
    
    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    Entity.add_status(:single)
    # def doc_id
    #   image && image.id
    # end

    # shortcut de relation
    def housing_offers
      Hermes::HousingOffer.where(:id.in => housing_entity_offers.pluck(:housing_offer_id))
    end

    # shortcut d'association
    def housing_offer_ids=(arr)
      previous_ids = housing_offers.pluck(:id).map(&:to_s)
      new_ids = (arr || []).map(&:to_s)
      (new_ids - previous_ids).each do |offer_id|
        Hermes::HousingEntityOffer.create(housing_offer_id: offer_id, entity_id: id)
      end
      Hermes::HousingEntityOffer.where(:housing_offer.in => (previous_ids - new_ids), entity_id: id).destroy
    end

    # shortcut de relation
    def housing_offer_ids
      housing_offers.pluck(:id)
    end

    # shortcut d'association
    def main_contact_id=(id)
      contact_group&.set(main_contact_id: id)
    end

    # shortcut d'accès au group
    def supervisor_group
      Group.find_by(id: supervisor_group_id)
    end

    # shortcut d'accès au group
    def manager_group
      Group.find_by(id: manager_group_id)
    end

    # gestionnaire de mise à jour du total des points dépensés et la balance
    def update_spent_points
      self.total_spent = SPENT_ON_FIELDS.inject(0) do |sum, spent_on_field|
        sum + send(spent_on_field) || 0
      end
      self.points_balance = annual_stock - total_spent
    end

    # gestionnaire de génération des groupes essentiels aux ACL
    def create_and_set_own_groups
      group = Group.find_or_create_by(name: "entity_#{siret}")
      self.group_id = group.id
      supervisor_group = Group.find_or_create_by(name: "entity_supervisor_#{siret}")
      self.supervisor_group_id = supervisor_group.id
      manager_group = Group.find_or_create_by(name: "entity_manager_#{siret}")
      self.manager_group_id = manager_group.id
      readers.concat(["g_#{group.id}", "g_#{supervisor_group.id}"])
      writers << "g_#{manager_group.id}"
      [supervisor_group, manager_group, group].each do |target_group|
        target_group.set(
          writers: (target_group.writers + [user_id]).uniq.compact,
          updated_at: DateTime.current
        )
      end
    end

    def before_create_actions
      create_and_set_own_groups
      update_spent_points
    end

    def before_actions
      update_spent_points if SPENT_ON_FIELDS.any? do |field|
        send("#{field}_changed?")
      end
    end

    def after_actions
      check_status_changed
    end
  end
end
