# frozen_string_literal: trues

module Hermes
  # HousingIntention model
  class HousingApplication
    include Hermes::Concerns::CommonFieldsConcern
    include Hermes::Concerns::Status
    include Mongoid::Attributes::Dynamic

    field :applicated_at, type: DateTime
    field :effort_rate, type: Float
    field :epci, type: Integer
    field :is_active, type: Boolean, default: true
    field :max_date, type: DateTime
    field :visit_conducted_date, type: DateTime
    field :last_xml_validation_report, type: String
    field :sopra_code, type: String

    common_indexes *%i[
      visit_status
      visit_status_id
      visit_status_updated_at
    ]

    belongs_to :housing_offer, index: true, class_name: 'Hermes::HousingOffer'
    belongs_to :housing_request, index: true, class_name: 'Hermes::HousingRequest'

    validates_presence_of :housing_request_id, :housing_offer
    validates :status, inclusion: { in: Hermes::Statuses::HousingApplication.list[:commission], allow_blank: true }
    validates :visit_status, inclusion: { in: Hermes::Statuses::HousingApplication.list[:visit], allow_blank: true }

    writer_roles 'Superadmin', 'Administrator'
    reader_roles 'Manager'
    owner_field :user_id

    index(applicated_at: 1)
    index(effort_rate: 1)
    index(epci: 1)
    index(is_active: 1)
    index(max_date: 1)

    before_validation :before_validation_actions
    after_save :after_save_actions
    after_create :after_create_actions
    after_update :after_actions
    after_destroy :after_destroy_actions

    scope :selected_by_offer, ->(housing_offer_id) {
      where(housing_offer_id: housing_offer_id, status: "TRANSMIS BAILLEUR")
    }

    HousingApplication.add_status(:single)
    HousingApplication.add_status(:single, "visit_status")

    def to_json(options={})
      options[:except] ||= :sopra_code
      super
    end

    def as_json(options={})
      options[:except] ||= :sopra_code
      super
    end

    def applicant
      housing_request&.main_applicant
    end

    def applicant_id
      applicant&.id
    end

    def applicated_nb_do(dec_or_inc_sym)
      return unless housing_offer

      case dec_or_inc_sym
      when :dec
        housing_offer.inc_applicated_nb(-1)
      when :inc
        housing_offer.inc_applicated_nb(1)
      end
    end

    def handle_applicated_nb
      if is_active
        applicated_nb_do :inc
      else
        applicated_nb_do :dec
      end
    end

    def handle_status_changed
      if status&.upcase&.include?('DESISTEMENT') && visit_status&.upcase == 'VISITE EFFECTUEE'
        applicant.canceled_application_nb += 1
        applicant.save
        housing_request.update_applicant_eligbility
      end
    end

    def handle_visit_conducted_date
      if visit_conducted_date
        if visit_conducted_date.past?
          self.set(visit_status: 'Visite effectuée') if  visit_status.blank?
        else
          self.set(visit_status: 'Visite proposée') if  visit_status.blank?
        end
      end
    end

    def update_status
      Hermes::BoClient::UpdateStatusTask.new(
        {
          housing_application_id: id,
          housing_request_id: housing_request_id,
          housing_offer_id: housing_offer_id,
          code_evenement: sopra_code
        }
      ).execute
    end

    def before_validation_actions
      if status_changed?
        self.status_updated_at = DateTime.current
        self.sopra_code = Hermes::Statuses::HousingApplication.sopra_code(status)
        self.status = Hermes::Statuses::HousingApplication.correspondence(status || '')
      end

      if visit_status_changed?
        self.visit_status_updated_at = DateTime.current 
        self.visit_status = I18n.transliterate(visit_status || '').upcase
      end
      update_right
    end

    def update_right
      lessor_user_id = housing_offer&.lessor&.user_id&.to_s
      main_applicant = housing_request.main_applicant
      reader_inst = [self, housing_request, main_applicant]
      if lessor_user_id
        if disabled_or_deleted_status?
          reader_inst.each { |inst| inst.readers.delete(lessor_user_id) }
          housing_request.applicant_ids.each do |co_id|
            coapplicant = Hermes::Applicant.find co_id
            coapplicant.readers.delete(lessor_user_id)
            coapplicant.set(readers: coapplicant.readers)
          end
        else
          reader_inst.each { |inst| inst.readers.push(lessor_user_id) }
          housing_request.applicant_ids.each do |co_id|
            coapplicant = Hermes::Applicant.find co_id
            if coapplicant
              coapplicant.readers.push(lessor_user_id)
              coapplicant.readers.uniq!
              coapplicant.set(readers: coapplicant.readers)
            end
          end
        end
        reader_inst.each do |inst|
          inst.readers.uniq!
          inst.set(readers: main_applicant.readers)
        end
      end
    end

    def after_save_actions
      housing_request.save
      update_status if status_changed?
    end

    def after_actions
      handle_visit_conducted_date
      handle_applicated_nb if is_active_changed?
      handle_status_changed if visit_status_changed? || status_changed?
    end

    def after_create_actions
      applicated_nb_do :inc if is_active
      housing_request.set(last_application_date: created_at, updated_at: DateTime.current)

      Hermes::BoClient::ApplicationTask.new(
        {
          housing_application_id: id,
          housing_request_id: housing_request_id,
          housing_offer_id: housing_offer_id,
        }
      ).execute
    end

    def after_destroy_actions
      housing_request.save
      applicated_nb_do :dec
    end

    def disabled_or_deleted_status?
      status&.upcase&.include?('ANNULEE') || status&.upcase&.include?('DESISTEMENT')
    end
  end
end
